package org.embulk.parser;

import org.embulk.config.Config;
import org.embulk.config.ConfigDefault;
import org.embulk.config.ConfigDiff;
import org.embulk.config.ConfigSource;
import org.embulk.config.Task;
import org.embulk.config.TaskSource;
import org.embulk.spi.ParserPlugin;
import org.embulk.spi.FileInput;
import org.embulk.spi.PageOutput;
import org.embulk.spi.Schema;
import org.embulk.spi.SchemaConfig;

import org.embulk.spi.Exec;
import org.embulk.spi.PageBuilder;
import org.embulk.spi.util.FileInputInputStream;
import org.embulk.spi.ColumnConfig;
import java.io.IOException;
import java.util.Arrays;
import java.util.ArrayList;
import org.apache.commons.codec.binary.Base64;

import static org.embulk.spi.type.Types.STRING;

import org.slf4j.Logger;


public class NoneParserPlugin
        implements ParserPlugin
{
    static int MAX_NAME_LENGTH = 255;

    public interface PluginTask
            extends Task //, LineDecoder.DecoderTask //, TimestampParser.Task
    {
        @Config("column_name")
        @ConfigDefault("\"payload\"")
        public String getColumnName();
    }

    private final Logger log;

    public NoneParserPlugin()
    {
        this.log = Exec.getLogger(NoneParserPlugin.class);
    }

    @Override
    public void transaction(ConfigSource config, ParserPlugin.Control control)
    {
        PluginTask task = config.loadConfig(PluginTask.class);
        ArrayList<ColumnConfig> columns = new ArrayList<ColumnConfig>();
        final String columnName = task.getColumnName();

        columns.add(new ColumnConfig(columnName, STRING, config));
        columns.add(new ColumnConfig("tag", STRING, config)); 

        Schema schema = new SchemaConfig(columns).toSchema();
        control.run(task.dump(), schema);
    }

    @Override
    public void run(TaskSource taskSource, Schema schema,
            FileInput input, PageOutput output)
    {
        PluginTask task = taskSource.loadTask(PluginTask.class);
        FileInputInputStream dataIn = new FileInputInputStream(input);
        PageBuilder pageBuilder = new PageBuilder(Exec.getBufferAllocator(), schema, output);
        int chunksize = 1024 * 1024 * 10;
        
        while( input.nextFile() ){
            byte[] pathBytesArray = new byte[MAX_NAME_LENGTH];
            int i = 0;
            int c;
            for (; i < MAX_NAME_LENGTH; i++) {
                c = dataIn.read();
                if ( c == -1) {
                    break;
                } else if ( c == 0 ) {
                    // read empty bytes until MAX_NAME_LENGTH;
                    for (int j = i + 1; j < MAX_NAME_LENGTH; j++) {
                        dataIn.read();
                    }
                    break;
                }
                pathBytesArray[i] = (byte)c;
            }
            String path = new String(Arrays.copyOfRange(pathBytesArray, 0, i));
            int bytes_read = 0;
            while( bytes_read != -1 ) {
                byte bytesArray[] = new byte[chunksize];
                int offset = 0;
                while ( offset < chunksize ) {
                    bytes_read = dataIn.read(bytesArray, offset, chunksize);
                    if( bytes_read == -1 ) {
                      break;
                    }
                    offset += bytes_read;
                }

                log.info(offset +  path);
                pageBuilder.setString(0, Base64.encodeBase64String(Arrays.copyOfRange(bytesArray, 0, offset)));
                pageBuilder.setString(1, path); 
                pageBuilder.addRecord();
            }
        }
        pageBuilder.finish();
    }
}
